﻿using System.Windows;
using Infragistics.Olap;
using Infragistics.Olap.FlatData;
using PercentageAggregator.UI.Helper;
using Microsoft.Win32;
using System.IO;
using System;

namespace PercentageAggregator.UI
{
    /// <summary>
    ///     Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            var flatDataSource = BuildPivotGridFlatDataSource();

            pivotGrid.DataSource = flatDataSource;
            dataSelector.DataSource = flatDataSource;
        }

        private IOlapViewModel BuildPivotGridFlatDataSource()
        {
            var flatDataSource = new FlatDataSource
                {
                    ItemsSource = new SalesCollection(),
                    Cube = DataSourceBase.GenerateInitialCube(@"Sale"),
                    MeasureListLocation = MeasureListLocation.Columns,
                    DimensionsGenerationMode = DimensionsGenerationMode.Mixed,
                    ParentInFrontForColumns = true
                };

            var cubeMetadata = new CubeMetadata {DisplayName = @"Sample Cube", DataTypeFullName = typeof(Sale).FullName};

            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"NumberOfUnits",
                    DisplayName = @"Number of Units"
                });

            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"Product",
                    DisplayName = @"Product"
                });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"City",
                    DisplayName = @"City"
                });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"Seller",
                    DisplayName = @"Seller"
                });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"Date",
                    DisplayName = @"Date"
                });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"Quarter",
                    DisplayName = @"Quarter"
                });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"UnitPrice",
                    DisplayName = @"Unit Price"
                });

            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"Value",
                    DisplayName = @"Value"
                });

            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"AmountOfSale",
                    DimensionType = DimensionType.Measure,
                    DisplayName = @"Amount of Sale Sum",
                    DisplayFormat = @"{0:C2}",
                    Aggregator = new DoubleSumAggregator {Identity = @"DoubleSumAmountOfSale"}
                });

            var percentage = new PercentOfTotalAggregator();
            percentage.Parts.Add(new DoubleSumAggregator {Identity = @"DoubleSumAmountOfSale"});

            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    DimensionType = DimensionType.Measure,
                    DisplayName = @"Amount Of Sale Percentage",
                    Aggregator = percentage,
                    DisplayFormat = @"{0:f2}%"
                });

            flatDataSource.CubesSettings.Add(cubeMetadata);

            var sellerHierarchy = new HierarchyDescriptor<Sale>(p => p.Seller);
            sellerHierarchy.AddLevel(p =>
                                     @"All Sellers", 
                                     @"All Sellers"
                );
            sellerHierarchy.AddLevel(p =>
                                     p.Seller.Name,
                                     @"Seller Name"
                );
            flatDataSource.AddHierarchyDescriptor(sellerHierarchy);

            var productHierarchy = new HierarchyDescriptor<Sale>(p => p.Product);
            productHierarchy.AddLevel(p =>
                                      @"All Products",
                                      @"All Products"
                );
            productHierarchy.AddLevel(p =>
                                      p.Product.Name,
                                      @"Product Name"
                );
            flatDataSource.AddHierarchyDescriptor(productHierarchy);

            var locationHierarchy = new HierarchyDescriptor<Sale>(p => p.City);
            locationHierarchy.AddLevel(p =>
                                       @"All Locations",
                                       @"All Locations"
                );
            locationHierarchy.AddLevel(p =>
                                       p.City,
                                       @"City"
                );
            flatDataSource.AddHierarchyDescriptor(locationHierarchy);

            return flatDataSource;
        }

        private void Save_Click(object sender, RoutedEventArgs e)
        {
            DataSourceBase pivotGridDataSource = (DataSourceBase)pivotGrid.DataSource;//.SaveCustomizations();
            
            SaveFileDialog saveFileDialog = new SaveFileDialog()
            {
                DefaultExt = "xml",
                Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*",
                FilterIndex = 1
            };

            if (saveFileDialog.ShowDialog() == true)
            {
                using (Stream stream = saveFileDialog.OpenFile())
                {
                    pivotGridDataSource.SaveCustomizations(stream);
                }
            }
        }

        private void Load_Click(object sender, RoutedEventArgs e)
        {
           // ((DataSourceBase)pivotGrid.DataSource).LoadCustomizations(customizations, LoadCompletedAction);
            DataSourceBase pivotGridDataSource = (DataSourceBase)pivotGrid.DataSource;//.SaveCustomizations();

            OpenFileDialog openFileDialog = new OpenFileDialog()
            {
                Multiselect = false,
                Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*",
                FilterIndex = 1
            };

            if (openFileDialog.ShowDialog() != true)
                return;
            try
            {

                using (Stream stream = openFileDialog.OpenFile())

                {
                    pivotGridDataSource.LoadCustomizations(stream, LoadCompletedAction);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("Error: {0}", ex.Message));
            }
        }

        void LoadCompletedAction(string errorsLog)
        {
            if (!string.IsNullOrEmpty(errorsLog))
            {
                // there were errors - handle this accordingly
            }
        }
    }
}